Attribute VB_Name = "FileSM"
'  FILE:       FileSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    This module contains U/I specific functions
'              to display the File Open dialog
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "FileSM."

' Used to distinguish the method that the user chooses to
' enter the step text
Public Const gstrPreview = "Preview"
Public Const gstrEntry = "Entry"
Public Const gstrInline As String = "<<Inline>>"


Public Sub BrowseCompareOutput()
    ' Called when the user selects the Browse button in the
    ' Failure details frame

    Dim strStepTextFileName As String
    
    On Error GoTo BrowseCompareOutputErr
    gstrSource = mstrModuleName & "BrowseCompareOutput"
    
    ' Call CallFileDialog to display the open file dialog
    strStepTextFileName = CallFileDialog(gintOutputCompareFile, _
            frmSteps.txtFailureDetails.Text)
    
    ' Set the text in the failure details text box to the output
    ' compare file selected by the user
    frmSteps.txtFailureDetails = strStepTextFileName

    Exit Sub
    
BrowseCompareOutputErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    Call ShowError(errBrowseFailed)
    
End Sub
Public Sub BrowseStartDir()
    ' Called when the user selects the Browse button in the Start Directory frame

    On Error GoTo BrowseStartDirErr
    
    ' Call CallFileDialog to display the open file dialog
    frmSteps.txtStartDir = GetStartDir(frmSteps.txtStartDir.Text)

    Exit Sub
    
BrowseStartDirErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "BrowseStartDir"
    Call ShowError(errBrowseFailed)
    
End Sub

Public Sub BrowseStepText()
    ' Called when the user selects the Browse button in the
    ' Step Description frame

    Dim strStepTextFileName As String
    
    On Error GoTo BrowseStepTextErr
    
    ' Call CallFileDialog to display the open file dialog
    strStepTextFileName = CallFileDialog(gintStepTextFile, _
            frmSteps.txtStepTextFileName.Text)
    
    frmSteps.txtStepTextFileName = strStepTextFileName
    
    If Not StringEmpty(strStepTextFileName) Then
        strStepTextFileName = SubstituteParameters(strStepTextFileName, _
                    CLng(Trim$(frmSteps.lblWorkspaceId)))
        ' Call a procedure to display a preview of the file containing
        ' the step text selected by the user
        Call PreviewStepTextFile(strStepTextFileName, _
                frmSteps.txtStepText, frmSteps.lblStepText)
    End If
    
    Exit Sub
    
BrowseStepTextErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "BrowseStepText"
    Call ShowError(errBrowseFailed)
    
End Sub
Public Function BrowseImportFile() As String
    ' Prompts the user for a workspace file to import from
    ' Call CallFileDialog to display the open file dialog
    BrowseImportFile = CallFileDialog(gintImportFile)
    
End Function
Public Function BrowseExportFile() As String
    ' Prompts the user for a workspace file to Export to
    ' Call CallFileDialog to display the open file dialog
    BrowseExportFile = CallFileDialog(gintExportFile)
    
End Function

Public Function BrowseDBFileNew() As String
    ' Prompts the user for the workspace definition file to be created
    ' Call CallFileDialog to display the open file dialog
    BrowseDBFileNew = CallFileDialog(gintDBFileNew)
    
End Function

Public Sub PreviewStepTextFile(strFileName As String, _
    txtOutput As TextBox, lblOutput As Label)
    ' Displays the contents of the passed in file in the Step Text
    ' rich text box control

    Dim FileNum As Integer
    Dim strTemp As String
    
    On Error GoTo PreviewStepTextFileErr
    
    FileNum = FreeFile
    
    If Not StringEmpty(strFileName) Then
        
        Call SetKeywordText(txtOutput, strTag:=gstrPreview)
        
        Open strFileName For Input Access Read As FileNum
        
        Line Input #FileNum, strTemp ' Read line into variable.
        txtOutput.Text = strTemp
        
        Do While Not EOF(FileNum) ' Loop until end of file.
            txtOutput.Text = txtOutput.Text & vbCr & vbLf
            Line Input #FileNum, strTemp ' Read line into variable.
            txtOutput.Text = txtOutput.Text & strTemp
        Loop
        txtOutput.Locked = True
    
        ' Modify the caption of the text field so that the
        ' user knows what he's seeing
        lblOutput.Caption = "Previewing the contents of " & gstrSQ & strFileName & gstrSQ
    End If
    
    Close FileNum
    
    Exit Sub
    
PreviewStepTextFileErr:
    txtOutput.Text = gstrEmptyString
    lblOutput.Caption = "The &Step to be Executed"
    
    ' Log the error code raised by Visual Basic
    gstrSource = mstrModuleName & "PreviewStepTextFile"
    Call LogErrors(Errors)
    ' Display a message saying that the preview cannot be
    ' displayed
    Call ShowError(errUnableToPreviewFile)
    
    ' Make sure that the user can only enter a file name
    Call EnteringStepFile
    
End Sub
'Private Function ShowFileOpenDialog(ByVal strFilter As String, _
'        ByVal strDialogTitle As String, ByVal lngFlags As Long, _
'        ByVal strOldFile As String) As String
'    ' Sets the properties of the common dialog to the values passed
'    ' in and displays it
'    ' Returns the file name selected by the user
'    Dim strInitDir As String
'    Dim intPos As Integer
'    Dim opfile As o
'
'    On Error GoTo ShowFileOpenDialogErr
'
'    If Not StringEmpty(strOldFile) Then
'        intPos = InstrR(strOldFile, gstrFileSeparator)
'        If intPos > 0 Then
'            strInitDir = Left$(strOldFile, intPos - 1)
'        End If
'    End If
'
''    With frmMain.dlgCommonDialog
''
''        .DialogTitle = strDialogTitle
''        .Flags = lngFlags
''        .Filter = strFilter
''        ' We want an error to be generated when the user says
''        ' Cancel on the dialog!
''        .CancelError = True
''        .InitDir = strInitDir
''        .FileName = strOldFile
''        .ShowOpen
''        ShowFileOpenDialog = .FileName
''
''    End With
'
'
'    Exit Function
'
'ShowFileOpenDialogErr:
'    ' If the error is generated 'coz the user hit the Cancel
'    ' button, not really an error!
'    If Err.Number <> cdlCancel Then
'        ' Log the error code raised by Visual Basic
'        Call LogErrors(Errors)
'    End If
'    ' Reset the selection to the passed in file, if any
'    ShowFileOpenDialog = strOldFile
'
'End Function
'
