VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cConnDtl"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cConnDtl.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     Encapsulates the properties and methods of a connection.
'               Contains functions to insert, update and delete
'               connection_dtls records from the database.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit
Option Base 0

' Local variable(s) to hold property value(s)
Public WorkspaceId As Long
Public ConnNameId As Long
Public ConnName As String
Public ConnectionString As String
Public ConnType As ConnectionType
Public Position As Long
Public NodeDB As Database

Private mintOperation As Operation

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cConnDtl."

' The cSequence class is used to generate unique Connection identifiers
Private mConnectionSeq As cSequence

' The StringSM class is used to carry out string operations
Private mFieldValue As cStringSM

Private Sub AssignParameters(qyExec As DAO.QueryDef)
    ' Assigns values to the parameters in the querydef object
    ' The parameter names are cryptic to differentiate them from the field names.
    ' When the parameter names are the same as the field names, parameters in the where
    ' clause do not get created.
    
    Dim prmParam As DAO.Parameter
    
    On Error GoTo AssignParametersErr
    
    For Each prmParam In qyExec.Parameters
        Select Case prmParam.Name
            Case "[w_id]"
                prmParam.Value = WorkspaceId
            
            Case "[c_id]"
                prmParam.Value = ConnNameId
                
            Case "[c_name]"
                prmParam.Value = ConnName
                
            Case "[c_str]"
                prmParam.Value = ConnectionString
                
            Case "[c_type]"
                prmParam.Value = ConnType
                
            Case Else
                ' Write the parameter name that is faulty
                WriteError errInvalidParameter, mstrSource, prmParam.Name
                On Error GoTo 0
                Err.Raise errInvalidParameter, mstrModuleName & "AssignParameters", _
                        LoadResString(errInvalidParameter)
        End Select
    Next prmParam
    
    Exit Sub
        
AssignParametersErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errAssignParametersFailed, _
        mstrModuleName & "AssignParameters", LoadResString(errAssignParametersFailed)
    
End Sub

Public Function Clone() As cConnDtl

    ' Creates a copy of a given Connection
    
    Dim cCloneConn As cConnDtl
    
    On Error GoTo CloneErr
    
    Set cCloneConn = New cConnDtl
    
    ' Copy all the Connection properties to the newly created Connection
    cCloneConn.WorkspaceId = WorkspaceId
    cCloneConn.ConnNameId = ConnNameId
    cCloneConn.ConnName = ConnName
    cCloneConn.ConnectionString = ConnectionString
    cCloneConn.ConnType = ConnType
    cCloneConn.IndOperation = mintOperation
    cCloneConn.Position = Position
    
    ' And set the return value to the newly created Connection
    Set Clone = cCloneConn
    Set cCloneConn = Nothing
    
    Exit Function
        
CloneErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Clone"
    On Error GoTo 0
    Err.Raise vbObjectError + errCloneFailed, mstrSource, LoadResString(errCloneFailed)

End Function
Private Sub CheckDupConnectionName()
    ' Check if the Connection name already exists in the workspace
   
    Dim rstConnection As Recordset
    Dim strSql As String
    Dim qy As DAO.QueryDef

    On Error GoTo CheckDupConnectionNameErr
    mstrSource = mstrModuleName & "CheckDupConnectionName"
    
    ' Create a recordset object to retrieve the count of all Connections
    ' for the workspace with the same name
    strSql = "Select count(*) as Connection_count " & _
        " from " & TBL_CONNECTION_DTLS & _
        " where " & FLD_ID_WORKSPACE & " = [w_id]" & _
        " and " & FLD_CONN_DTL_CONNECTION_NAME & " = [c_name]" & _
        " and " & FLD_ID_CONN_NAME & " <> [c_id]"
    
    Set qy = dbsAttTool.CreateQueryDef(gstrEmptyString, strSql)
    Call AssignParameters(qy)
    
    Set rstConnection = qy.OpenRecordset(dbOpenForwardOnly)
    
    If rstConnection![Connection_count] > 0 Then
        rstConnection.Close
        qy.Close
        ShowError errDupConnDtlName
        On Error GoTo 0
        Err.Raise vbObjectError + errDupConnDtlName, _
            mstrSource, LoadResString(errDupConnDtlName)
    End If
    
    rstConnection.Close
    qy.Close

    Exit Sub
    
CheckDupConnectionNameErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "CheckDupConnectionName"
    On Error GoTo 0
    Err.Raise vbObjectError + errProgramError, _
        mstrSource, LoadResString(errProgramError)

End Sub
Public Property Let IndOperation(ByVal vdata As Operation)
    
    ' The valid operations are define in the cOperations
    ' class. Check if the operation is valid
    Select Case vdata
        Case QueryOp, InsertOp, UpdateOp, DeleteOp
            mintOperation = vdata
        
        Case Else
            BugAssert True
    End Select
    
End Property
Public Sub Validate()
    ' Each distinct object will have a Validate method which
    ' will check if the class properties are valid. This method
    ' will be used to check interdependant properties that
    ' cannot be validated by the let procedures.
    ' It should be called by the add and modify methods of the class
            
    If ConnName = gstrEmptyString Then
    
        ShowError errConnectionNameMandatory
        On Error GoTo 0
        ' Propogate this error back to the caller
        Err.Raise vbObjectError + errConnectionNameMandatory, _
            mstrSource, LoadResString(errConnectionNameMandatory)
    End If
    
    ' Raise an error if the Connection name already exists in the workspace
    Call CheckDupConnectionName

End Sub
Public Sub Add()

    Dim strInsert As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo AddErr
    
    ' Validate the record before trying to insert the record
    Call Validate
    
    ' Create a temporary querydef object
    strInsert = "insert into " & TBL_CONNECTION_DTLS & _
             "( " & FLD_ID_WORKSPACE & _
             ", " & FLD_ID_CONN_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_NAME & _
             ", " & FLD_CONN_DTL_CONNECTION_STRING & _
             ", " & FLD_CONN_DTL_CONNECTION_TYPE & " ) " & _
            " values ( [w_id], [c_id], " & _
            " [c_name], [c_str], [c_type] ) "

    Set qy = dbsAttTool.CreateQueryDef(gstrEmptyString, strInsert)
    
    ' Call a procedure to assign the Connection values
    Call AssignParameters(qy)
    
    qy.Execute dbFailOnError
    qy.Close

    Exit Sub
        
AddErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInsertFailed, _
        mstrModuleName & "Add", LoadResString(errInsertFailed)
    
End Sub
Public Sub Delete()

    Dim strDelete As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo DeleteErr
    
    strDelete = "delete from " & TBL_CONNECTION_DTLS & _
            " where " & FLD_ID_CONN_NAME & " = [c_id]"
    Set qy = dbsAttTool.CreateQueryDef(gstrEmptyString, strDelete)
    
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close
    
    Exit Sub
        
DeleteErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteFailed, _
            mstrModuleName & "Delete", LoadResString(errDeleteFailed)
    
End Sub

Public Sub Modify()

    Dim strUpdate As String
    Dim qy As QueryDef
            
    On Error GoTo ModifyErr
    
    ' Validate the updated values before trying to modify the db
    Call Validate
    
    ' Create a temporary querydef object with the modify string
    strUpdate = "update " & TBL_CONNECTION_DTLS & _
            " set " & FLD_ID_WORKSPACE & " = [w_id], " & _
            FLD_CONN_DTL_CONNECTION_NAME & " = [c_name], " & _
            FLD_CONN_DTL_CONNECTION_STRING & " = [c_str], " & _
            FLD_CONN_DTL_CONNECTION_TYPE & " = [c_type] " & _
            " where " & FLD_ID_CONN_NAME & " = [c_id]"
    Set qy = dbsAttTool.CreateQueryDef(gstrEmptyString, strUpdate)
    
    ' Call a procedure to assign the Connection values to the
    ' querydef object
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close

    Exit Sub
        
ModifyErr:

    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errModifyFailed, _
        mstrModuleName & "Modify", LoadResString(errModifyFailed)
    
End Sub
Public Property Get NextIdentifier() As Long
    
    Dim lngNextId As Long

    On Error GoTo NextIdentifierErr
    
    ' Retrieve the next identifier using the sequence class
    Set mConnectionSeq = New cSequence
    Set mConnectionSeq.IdDatabase = dbsAttTool
    mConnectionSeq.IdentifierColumn = FLD_ID_CONN_NAME
    lngNextId = mConnectionSeq.Identifier
    Set mConnectionSeq = Nothing
    
    NextIdentifier = lngNextId
    Exit Property
    
NextIdentifierErr:
    LogErrors Errors
    On Error GoTo 0
    Err.Raise vbObjectError + errIdGetFailed, _
        mstrModuleName & "NextIdentifier", LoadResString(errIdGetFailed)
    
End Property
Public Property Get IndOperation() As Operation

    IndOperation = mintOperation
    
End Property

Private Sub Class_Initialize()

    Set mFieldValue = New cStringSM
    
    ' Initialize the operation indicator variable to Query
    ' It will be modified later by the collection class when
    ' inserts, updates or deletes are performed
    mintOperation = QueryOp

    ConnType = giDefaultConnType
    
End Sub

Private Sub Class_Terminate()

    Set mFieldValue = Nothing
    
End Sub
