VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cInstance"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cInstance.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     Encapsulates the properties and methods of an instance.
'               An instance is created when a step is executed for a
'               particular iterator value (if applicable) at 'run' time.
'               Contains functions to determine if an instance is running,
'               complete, and so on.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this class
Private Const mstrModuleName As String = "cInstance."
Private mstrSource As String

Private mcStep As cStep
Public Key As String ' Node key for the step being executed
Public InstanceId As Long
Public ParentInstanceId As Long ' The parent instance
Private mblnNoMoreToStart As Boolean
Private mblnComplete As Boolean
Public StartTime As Currency
Public EndTime As Currency
Public ElapsedTime As Currency
Private mintStatus As InstanceStatus
Public DegreeParallelism As Integer
Private mcIterators As cRunColIt

' A collection of all the sub-steps for this step
Private mcSubSteps As cSubSteps
Public Sub UpdateStartTime(lStepId As Long, Optional ByVal StartTm As Currency = gdtmEmpty, _
        Optional ByVal EndTm As Currency = gdtmEmpty, _
        Optional ByVal Elapsed As Currency = 0)
    ' We do not maintain start and end timestamps for the constraint
    ' of a step. Hence we check if the process that just started/
    ' terminated is the worker step that is being executed. If so,
    ' we update the start/end time and status on the instance record.
    
    BugAssert (StartTm <> gdtmEmpty) Or (EndTm <> gdtmEmpty), "Mandatory parameter missing."
    
    ' Make sure that we are executing the actual step and not
    ' a pre or post-execution constraint
    If mcStep.StepId = lStepId Then
        If StartTm <> 0 Then
            StartTime = StartTm
            mintStatus = gintRunning
        Else
            EndTime = EndTm
            ElapsedTime = Elapsed
            mintStatus = gintComplete
        End If
    End If
    
End Sub
Public Function ValidForIteration(cParentInstance As cInstance, _
        ByVal intConsType As ConstraintType) As Boolean
    ' Returns true if the instance passed in is the first or
    ' last iteration for the step, depending on the constraint type
    
    Dim cSubStepRec As cSubStep
    Dim vntIterators As Variant
    
    On Error GoTo ValidForIterationErr
    
    If cParentInstance Is Nothing Then
        ' This will only be true for the dummy instance, which
        ' cannot have any iterators defined for it
        ValidForIteration = True
        Exit Function
    End If
    
    vntIterators = mcStep.Iterators
    
    If Not StringEmpty(mcStep.IteratorName) And Not IsEmpty(vntIterators) Then

        Set cSubStepRec = cParentInstance.QuerySubStep(mcStep.StepId)
        
        If intConsType = gintPreStep Then
            ' Pre-execution constraints will only be executed
            ' before the first iteration
            If cSubStepRec.LastIterator.IteratorType = gintValue Then
                ValidForIteration = (cSubStepRec.LastIterator.Sequence = _
                        gintMinIteratorSequence)
            Else
                ValidForIteration = (cSubStepRec.LastIterator.Value = _
                        cSubStepRec.LastIterator.RangeFrom)
            End If
        Else
            ' Post-execution constraints will only be executed
            ' after the last iteration - check if there are any
            ' pending iterations
            ValidForIteration = cSubStepRec.NextIteration(mcStep) Is Nothing
        End If
    Else
        ValidForIteration = True
    End If
        
    Exit Function
    
ValidForIterationErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    mstrSource = mstrModuleName & "ValidForIteration"
    Err.Raise vbObjectError + errExecInstanceFailed, _
            mstrSource, LoadResString(errExecInstanceFailed)
    
End Function

Public Sub CreateSubStep(cSubStepDtls As cStep, RunParams As cArrParameters)

    Dim cNewSubStep As cSubStep
    
    On Error GoTo CreateSubStepErr

    Set cNewSubStep = New cSubStep
    
    cNewSubStep.StepId = cSubStepDtls.StepId
    cNewSubStep.TasksComplete = 0
    cNewSubStep.TasksRunning = 0
    
    ' Initialize the iterator for the instance
    Set cNewSubStep.LastIterator = New cRunItDetails
    Call cNewSubStep.InitializeIt(cSubStepDtls, RunParams)
    
    ' Add add the substep to the collection
    mcSubSteps.Add cNewSubStep

    Set cNewSubStep = Nothing
    
    Exit Sub
    
CreateSubStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    mstrSource = mstrModuleName & "CreateSubStep"
    Err.Raise vbObjectError + errProgramError, mstrSource, _
        LoadResString(errProgramError)
    
End Sub

Public Function QuerySubStep(ByVal SubStepId As Long) As cSubStep
    ' Retrieves the sub-step record for the passed in sub-step id
    
    Dim lngIndex As Long

    On Error GoTo QuerySubStepErr
    
    ' Find the sub-step node with the matching step id
    For lngIndex = 0 To mcSubSteps.Count - 1
        If mcSubSteps(lngIndex).StepId = SubStepId Then
            Set QuerySubStep = mcSubSteps(lngIndex)
            Exit For
        End If
    Next lngIndex
    
    Exit Function
    
QuerySubStepErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    mstrSource = mstrModuleName & "QuerySubStep"
    Err.Raise vbObjectError + errNavInstancesFailed, _
            mstrSource, LoadResString(errNavInstancesFailed)
    
End Function
Public Property Let AllStarted(ByVal vdata As Boolean)
    
    'bugmessage "Set All Started to " & vData & " for : " & _
        mstrKey
        
    mblnNoMoreToStart = vdata

End Property
Public Property Get AllStarted() As Boolean

    AllStarted = mblnNoMoreToStart
    
End Property
Public Property Let AllComplete(ByVal vdata As Boolean)
    
    'bugmessage "Set All Complete to " & vData & " for : " & _
        mstrKey

    mblnComplete = vdata

End Property

Public Property Get AllComplete() As Boolean

    AllComplete = mblnComplete
    
End Property

Public Sub ChildExecuted(mlngStepId As Long)
    ' This procedure is called when a sub-step executes.
    
    Dim lngIndex As Long

    On Error GoTo ChildExecutedErr
    
    BugAssert mcStep.StepType = gintManagerStep
    
    For lngIndex = 0 To mcSubSteps.Count - 1
        If mcSubSteps(lngIndex).StepId = mlngStepId Then
            mcSubSteps(lngIndex).TasksRunning = _
                    mcSubSteps(lngIndex).TasksRunning + 1
'            BugMessage "Tasks Running for Step Id : " & _
'                    CStr(mcSubSteps(lngIndex).StepId) & _
'                    " Instance Id: " & InstanceId & _
'                    " = " & mcSubSteps(lngIndex).TasksRunning
            Exit For
        End If
    Next lngIndex
    
    If lngIndex > mcSubSteps.Count - 1 Then
        ' The child step wasn't found - raise an error
        On Error GoTo 0
        Err.Raise vbObjectError + errInvalidChild, mstrModuleName, _
                LoadResString(errInvalidChild)
    End If
    
    Exit Sub
    
ChildExecutedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInstanceOpFailed, mstrModuleName & "ChildExecuted", _
            LoadResString(errInstanceOpFailed)
    
End Sub
Public Sub ChildTerminated(mlngStepId As Long)
    ' This procedure is called when any sub-step process
    ' terminates. Note: The TasksComplete field will be
    ' updated only when all the instances for a sub-step
    ' complete execution.
    Dim lngIndex As Long
    
    On Error GoTo ChildTerminatedErr

    BugAssert mcStep.StepType = gintManagerStep
    
    For lngIndex = 0 To mcSubSteps.Count - 1
        
        If mcSubSteps(lngIndex).StepId = mlngStepId Then
            mcSubSteps(lngIndex).TasksRunning = _
                    mcSubSteps(lngIndex).TasksRunning - 1
'            BugMessage "Tasks Running for Step Id : " & _
'                    CStr(mcSubSteps(lngIndex).StepId) & _
'                    " Instance Id: " & InstanceId & _
'                    " = " & mcSubSteps(lngIndex).TasksRunning
            
            BugAssert mcSubSteps(lngIndex).TasksRunning >= 0, _
                    "Tasks running for " & CStr(mlngStepId) & _
                    " Instance Id " & InstanceId & " is less than 0."
            Exit For
        End If
    Next lngIndex
    
    If lngIndex > mcSubSteps.Count - 1 Then
        ' The child step wasn't found - raise an error
        On Error GoTo 0
        Err.Raise errInvalidChild, mstrModuleName & "ChildTerminated", _
                LoadResString(errInvalidChild)
    End If
    
    Exit Sub
    
ChildTerminatedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    mstrSource = mstrModuleName & "ChildTerminated"
    Err.Raise vbObjectError + errInstanceOpFailed, mstrSource, _
            LoadResString(errInstanceOpFailed)
    
End Sub
Public Sub ChildCompleted(mlngStepId As Long)
    ' This procedure is called when any a sub-step completes
    ' execution. Note: The TasksComplete field will be
    ' incremented.
    Dim lngIndex As Long
    
    On Error GoTo ChildCompletedErr

    BugAssert mcStep.StepType = gintManagerStep
    
    For lngIndex = 0 To mcSubSteps.Count - 1
        BugAssert mcSubSteps(lngIndex).TasksComplete >= 0, _
                "Tasks complete for " & CStr(mcSubSteps(lngIndex).StepId) & _
                " Instance Id " & InstanceId & " is less than 0."
        
        If mcSubSteps(lngIndex).StepId = mlngStepId Then
            mcSubSteps(lngIndex).TasksComplete = _
                    mcSubSteps(lngIndex).TasksComplete + 1
'            BugMessage "Tasks Complete for Step Id : " & _
'                    CStr(mcSubSteps(lngIndex).StepId) & _
'                    " Instance Id: " & InstanceId & _
'                    " = " & mcSubSteps(lngIndex).TasksComplete
            Exit For
        End If
    Next lngIndex
    
    If lngIndex > mcSubSteps.Count - 1 Then
        ' The child step wasn't found - raise an error
        On Error GoTo 0
        Err.Raise errInvalidChild, mstrModuleName, _
                LoadResString(errInvalidChild)
    End If
    
    Exit Sub
    
ChildCompletedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInstanceOpFailed, mstrModuleName & "ChildCompleted", _
            LoadResString(errInstanceOpFailed)
    
End Sub
Public Sub ChildDeleted(mlngStepId As Long)
    ' This procedure is called when a sub-step needs to be re-executed
    ' Note: The TasksComplete field is decremented. We needn't worry about
    ' the TasksRunning field since no steps are currently running.
    Dim lngIndex As Long
    
    On Error GoTo ChildDeletedErr

    BugAssert mcStep.StepType = gintManagerStep
    
    For lngIndex = 0 To mcSubSteps.Count - 1
        
        If mcSubSteps(lngIndex).StepId = mlngStepId Then
            mcSubSteps(lngIndex).TasksRunning = _
                    mcSubSteps(lngIndex).TasksRunning - 1
                    
            BugAssert mcSubSteps(lngIndex).TasksRunning >= 0, _
                    "Tasks running for " & CStr(mcSubSteps(lngIndex).StepId) & _
                    " Instance Id " & InstanceId & " is less than 0."
            Exit For
        End If
    Next lngIndex
    
    If lngIndex > mcSubSteps.Count - 1 Then
        ' The child step wasn't found - raise an error
        On Error GoTo 0
        Err.Raise errInvalidChild, mstrModuleName, _
                LoadResString(errInvalidChild)
    End If
    
    Exit Sub
    
ChildDeletedErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errInstanceOpFailed, mstrModuleName & "ChildDeleted", _
            LoadResString(errInstanceOpFailed)
    
End Sub
Private Sub RaiseErrForWorker()

    If mcStep.StepType <> gintManagerStep Then
        On Error GoTo 0
        mstrSource = mstrModuleName & "RaiseErrForWorker"
        Err.Raise vbObjectError + errInvalidForWorker, _
                mstrSource, _
                LoadResString(errInvalidForWorker)
    End If
    
End Sub

Public Property Get Step() As cStep

    Set Step = mcStep
    
End Property
Public Property Get Iterators() As cRunColIt

    Set Iterators = mcIterators
    
End Property
Public Property Get SubSteps() As cSubSteps

    Call RaiseErrForWorker
    
    Set SubSteps = mcSubSteps
    
End Property
Public Property Set Step(cRunStep As cStep)

    Set mcStep = cRunStep
    
End Property

Public Property Set Iterators(cIts As cRunColIt)

    Set mcIterators = cIts
    
End Property
Public Property Get IsPending() As Boolean
    ' Returns true if the step has any substeps that need
    ' execution
    Dim lngIndex As Long
    Dim lngRunning As Long

    Call RaiseErrForWorker
    
    If Not mblnComplete And Not mblnNoMoreToStart Then
        ' Get a count of all the substeps that are already being
        ' executed
        lngRunning = 0
        For lngIndex = 0 To mcSubSteps.Count - 1
            lngRunning = lngRunning + mcSubSteps(lngIndex).TasksRunning
        Next lngIndex
        
        IsPending = (lngRunning < DegreeParallelism)
    Else
        ' This should be sufficient to prove that there r no
        ' more sub-steps to be executed.
        ' mblnComplete: Handles the case where all steps have
        ' been executed
        ' mblnNoMoreToStart: Handles the case where the step
        ' has a degree of parallelism greater than the total
        ' number of sub-steps available to execute
        IsPending = False
    End If
    
End Property
Public Property Get IsRunning() As Boolean
    ' Returns true if the any one of the substeps is still
    ' executing
    Dim lngIndex As Long

    Call RaiseErrForWorker
    
    IsRunning = False
    
    ' If a substep has no currently executing tasks and
    ' the tasks completed is greater than zero, then we can
    ' assume that it has completed execution (otherwise we
    ' would've run a new task the moment one completed!)
    For lngIndex = 0 To mcSubSteps.Count - 1
        If mcSubSteps(lngIndex).TasksRunning > 0 Then
            IsRunning = True
            Exit For
        End If
    Next lngIndex
    
End Property
Public Property Get TotalRunning() As Long
    ' Returns the total number of substeps that are executing
    Dim lngTotalProcesses As Long
    Dim lngIndex As Long

    Call RaiseErrForWorker
    
    lngTotalProcesses = 0
    For lngIndex = 0 To mcSubSteps.Count - 1
        BugAssert mcSubSteps(lngIndex).TasksRunning >= 0, _
                "Tasks running for " & CStr(mcSubSteps(lngIndex).StepId) & _
                " is less than 0."
        lngTotalProcesses = lngTotalProcesses + mcSubSteps(lngIndex).TasksRunning
    Next lngIndex
    
    TotalRunning = lngTotalProcesses
End Property
Public Property Get RunningForStep(lngSubStepId As Long) As Long
    ' Returns the total number of instances of the substep
    ' that are executing
    Dim lngIndex As Long

    Call RaiseErrForWorker
    
    For lngIndex = 0 To mcSubSteps.Count - 1
        BugAssert mcSubSteps(lngIndex).TasksRunning >= 0, _
                "Tasks running for " & CStr(mcSubSteps(lngIndex).StepId) & _
                " is less than 0."
        
        If mcSubSteps(lngIndex).StepId = lngSubStepId Then
            RunningForStep = mcSubSteps(lngIndex).TasksRunning
            Exit For
        End If
    Next lngIndex
    
    If lngIndex > mcSubSteps.Count - 1 Then
        ' The child step wasn't found - raise an error
        On Error GoTo 0
        Err.Raise errInvalidChild, mstrSource, _
                LoadResString(errInvalidChild)
    End If

End Property

Public Property Let Status(ByVal vdata As InstanceStatus)
    
    mintStatus = vdata
            
End Property

Public Property Get Status() As InstanceStatus
    
    Status = mintStatus

End Property
Private Sub Class_Initialize()

    Set mcSubSteps = New cSubSteps
    
    mblnNoMoreToStart = False
    mblnComplete = False
    StartTime = gdtmEmpty
    EndTime = gdtmEmpty
    
End Sub

Private Sub Class_Terminate()

    mcSubSteps.Clear
    Set mcSubSteps = Nothing

End Sub
