VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cNavTree"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cNavTree.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     This module contains functions to navigate a tree.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' The tree being navigated
Private mtreTree As TreeView

' The root node for the branch
Private mstrRootNode As String

' The current node in the branch
Private mstrCurrentNode As String

' The collection of traversed nodes
Private mstrNodes() As String

' A count of all traversed nodes
Private mintNodeCount As Integer

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cNavTree."

' The direction in which to travel
' Can take one of the values in the enum, DirectionOfTraversal
Private mintDirection As Integer
Public Enum DirectionOfTraversal
    gintTopDown = 0
    gintBottomUp
End Enum
Public Property Let RootNode(ByVal vdata As String)
    
    ' The root node at which traversal should end - all nodes
    ' under the root node will be traversed. Must be a valid
    ' node in the tree
    
    Dim nodDummy As comctllib.Node
    
    On Error GoTo RootNodeErr
    mstrSource = mstrModuleName & "RootNode"
    
    If mtreTree Is Nothing Then
        On Error GoTo 0
        Err.Raise vbObjectError + errInitializeTree, _
                mstrSource, _
                LoadResString(errInitializeTree)
    End If
    
    ' Will raise an error if the root node is not valid
    Set nodDummy = mtreTree.Nodes(vdata)
    
    mstrRootNode = vdata

    Exit Property
        
RootNodeErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "RootNode"
    On Error GoTo 0
    Err.Raise vbObjectError + errRootNodeFailed, _
            mstrSource, _
            LoadResString(errRootNodeFailed)

End Property
Public Property Let Direction(ByVal vdata As DirectionOfTraversal)
    
    mstrSource = mstrModuleName & "StepType"
    
    Select Case vdata
        Case gintTopDown, gintBottomUp
            mintDirection = vdata
            
        Case Else
            ' Set the default value to Bottom Up traversal
            mintDirection = gintBottomUp
            WriteError errDirectionInvalid, mstrSource, _
                    Str(mintDirection)
            
    End Select
    
End Property

Public Property Set Tree(ByVal vdata As TreeView)
    
    Set mtreTree = vdata

End Property

Public Property Get NextNode() As String
    
    ' This is the core procedure in the class - it traverses
    ' through the tree starting at the root node and returns
    ' the key for each node - it returns an empty string when
    ' done
    Dim nodCurrent As Node
    
    On Error GoTo NextNodeErr
    mstrSource = mstrModuleName & "NextNode"
    
    If StringEmpty(mstrCurrentNode) Then  'first time through
        If StringEmpty(mstrRootNode) Then
            On Error GoTo 0
            Err.Raise errRootNodeKeyInvalid, mstrSource, _
                    LoadResString(errRootNodeKeyInvalid)
        Else
            If mtreTree.Nodes(mstrRootNode).Children > 0 Then
                mstrCurrentNode = mtreTree.Nodes(mstrRootNode).Child.Key
            Else
                NextNode = mstrCurrentNode
                Exit Property
            End If
        End If
    End If
    
    ' It utilizes a list of previously navigated nodes to
    ' determine whether to branch off in a given direction
    
    Set nodCurrent = mtreTree.Nodes(mstrCurrentNode)
        
    Do While nodCurrent.Key <> mstrRootNode
        
        ' The difference when we traverse the nodes top down
        ' and bottom up will be that in the former case, we
        ' add the nodes to the tree before we drill down to
        ' the children nodes.
        If mintDirection = gintTopDown And _
                Not InNavList(nodCurrent.Key) Then
            ' Add the current node to the list, if it does not
            ' exist - return this node as the next node in the
            ' tree
            mstrCurrentNode = nodCurrent.Key
            AddToArray mstrCurrentNode
            NextNode = mstrCurrentNode
            Exit Do
        End If
        
        ' The node already exists in the list
        If nodCurrent.Children > 0 Then
            ' Go to the child node if one exists and it is
            ' not in the list
            If Not InNavList(nodCurrent.Child.Key) Then
                Set nodCurrent = nodCurrent.Child
            ElseIf mintDirection = gintBottomUp And _
                    Not InNavList(nodCurrent.Key) Then
                ' Add the current node to the list, if it does not
                ' exist - return this node as the next node in the
                ' tree
                mstrCurrentNode = nodCurrent.Key
                AddToArray mstrCurrentNode
                NextNode = mstrCurrentNode
                Exit Do
            ' Go to the sibling node, if it exists
            ElseIf nodCurrent.Index <> nodCurrent.LastSibling.Index Then
                Set nodCurrent = nodCurrent.Next
            Else
                ' Go to the parent node
                Set nodCurrent = nodCurrent.Parent
            End If
        ElseIf mintDirection = gintBottomUp And _
                Not InNavList(nodCurrent.Key) Then
            ' Add the current node to the list, if it does not
            ' exist - return this node as the next node in the
            ' tree
            mstrCurrentNode = nodCurrent.Key
            AddToArray mstrCurrentNode
            NextNode = mstrCurrentNode
            Exit Do
        ' This repetition can be attributed solely to the
        ' absence of a continue keyword in the language!
        ' Go to the sibling node, if it exists
        ElseIf nodCurrent.Index <> nodCurrent.LastSibling.Index Then
            Set nodCurrent = nodCurrent.Next
        Else
            ' Go to the parent node
            Set nodCurrent = nodCurrent.Parent
        End If
            
    Loop
    
    ' If the exit condition was reached, we have completed
    ' traversing through the entire tree
    If nodCurrent.Key = mstrRootNode Then
        NextNode = gstrEmptyString
    End If
    
    Exit Property
    
NextNodeErr:
'    ' Log the error code raised by Visual Basic
    LogErrors Errors
    mstrSource = mstrModuleName & "NextNode"
    On Error GoTo 0
    Err.Raise vbObjectError + errNextNodeFailed, _
            gstrSource, _
            LoadResString(errNextNodeFailed)

End Property
Private Function InNavList(strToSearch As String) As Boolean

    ' Returns true if the passed in string exists in the list
    ' of node keys, else it returns false
    
    Dim intIndex As Integer
    
    On Error GoTo InNavListErr
    mstrSource = mstrModuleName & "InNavList"
    
    InNavList = False
    
    For intIndex = 0 To mintNodeCount - 1
        If mstrNodes(intIndex) = strToSearch Then
            InNavList = True
            Exit Function
        End If
    Next intIndex
    
    Exit Function
    
InNavListErr:
    ' Log the error code raised by Visual Basic
    LogErrors Errors
    mstrSource = mstrModuleName & "InNavList"
    On Error GoTo 0
    Err.Raise vbObjectError + errInListFailed, _
            gstrSource, _
            LoadResString(errInListFailed)

End Function

Private Sub AddToArray(strToAdd As String)

    ' Adds the passed in string to the list of node keys

    On Error GoTo AddToArrayErr
    
    ' Increase the array dimension by 1
    ReDim Preserve mstrNodes(mintNodeCount)
    
    ' and add the string to it
    mstrNodes(mintNodeCount) = strToAdd

    mintNodeCount = mintNodeCount + 1
    
    Exit Sub
    
AddToArrayErr:
    ' Log the error code raised by Visual Basic
    LogErrors Errors
    mstrSource = mstrModuleName & "AddToArray"
    On Error GoTo 0
    Err.Raise vbObjectError + errAddToArrayFailed, _
            gstrSource, _
            LoadResString(errAddToArrayFailed)

End Sub

Private Sub Class_Initialize()

    mintNodeCount = 0
    mstrCurrentNode = gstrEmptyString
    mintDirection = gintBottomUp
    
End Sub

Private Sub Class_Terminate()

    ReDim mstrNodes(0)
    
End Sub
