VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cParameter"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'  FILE:        cParameter.cls
'               Microsoft TPC-H Kit Ver. 2.7.0-1005
'               Copyright Microsoft, 2008
'               All Rights Reserved
'
'
'  PURPOSE:     Encapsulates the properties and methods of a parameter.
'               Contains functions to insert, update and delete
'               workspace_parameters records from the database.
'  Contact:     Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit
Option Base 0

' Local variable(s) to hold property value(s)
Private mlngWorkspaceId As Long
Private mlngParameterId As Long
Private mstrParameterName As String
Private mstrParameterValue As String
Private mstrDescription As String
Private mintParameterType As Integer
Private mdbsStepMaster As Database
Private mintOperation As Operation
Private mlngPosition As Long

' Used to indicate the source module name when errors
' are raised by this class
Private mstrSource As String
Private Const mstrModuleName As String = "cParameter."

' The cSequence class is used to generate unique parameter identifiers
Private mParameterSeq As cSequence

' The StringSM class is used to carry out string operations
Private mFieldValue As cStringSM

' Parameter types
Public Enum ParameterType
    gintParameterGeneric = 0
    gintParameterConnect
    gintParameterApplication
    gintParameterBuiltIn
End Enum


Private Sub AssignParameters(qyExec As DAO.QueryDef)
    ' Assigns values to the parameters in the querydef object
    ' The parameter names are cryptic to make them different
    ' from the field names. When the parameter names are
    ' the same as the field names, parameters in the where
    ' clause do not get created.
    
    Dim prmParam As DAO.Parameter
    
    On Error GoTo AssignParametersErr
    
    For Each prmParam In qyExec.Parameters
        Select Case prmParam.Name
            Case "[w_id]"
                prmParam.Value = mlngWorkspaceId
            
            Case "[p_id]"
                prmParam.Value = mlngParameterId
                
            Case "[p_name]"
                prmParam.Value = mstrParameterName
                
            Case "[p_value]"
                prmParam.Value = mstrParameterValue
                
            Case "[desc]"
                prmParam.Value = mstrDescription
                
            Case "[p_type]"
                prmParam.Value = mintParameterType
                
            Case Else
                ' Write the parameter name that is faulty
                WriteError errInvalidParameter, mstrSource, _
                        prmParam.Name
                On Error GoTo 0
                Err.Raise errInvalidParameter, mstrModuleName & "AssignParameters", _
                        LoadResString(errInvalidParameter)
        End Select
    Next prmParam
    
'    qyExec.Parameters("w_id").Value = mlngWorkspaceId
'    qyExec.Parameters("p_id").Value = mlngParameterId
'    qyExec.Parameters("p_name").Value = mstrParameterName
'    qyExec.Parameters("p_value").Value = mstrParameterValue
'
    
    Exit Sub
        
AssignParametersErr:

    mstrSource = mstrModuleName & "AssignParameters"
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errAssignParametersFailed, _
        mstrSource, LoadResString(errAssignParametersFailed)
    
End Sub

Public Property Let Position(ByVal RHS As Long)

    mlngPosition = RHS
    
End Property

Public Property Get Position() As Long

    Position = mlngPosition
    
End Property

Public Function Clone() As cParameter

    ' Creates a copy of a given parameter
    
    Dim cCloneParam As cParameter
    
    On Error GoTo CloneErr
    mstrSource = mstrModuleName & "Clone"
    
    Set cCloneParam = New cParameter
    
    ' Copy all the parameter properties to the newly
    ' created parameter
    Set cCloneParam.NodeDB = mdbsStepMaster
    cCloneParam.WorkspaceId = mlngWorkspaceId
    cCloneParam.ParameterId = mlngParameterId
    cCloneParam.ParameterName = mstrParameterName
    cCloneParam.ParameterValue = mstrParameterValue
    cCloneParam.Description = mstrDescription
    cCloneParam.ParameterType = mintParameterType
    cCloneParam.IndOperation = mintOperation
    cCloneParam.Position = mlngPosition
    
    ' And set the return value to the newly created parameter
    Set Clone = cCloneParam
    Set cCloneParam = Nothing
    
    Exit Function
        
CloneErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Clone"
    On Error GoTo 0
    Err.Raise vbObjectError + errCloneFailed, _
            mstrSource, LoadResString(errCloneFailed)

End Function
Public Property Set NodeDB(vdata As Database)

    Set mdbsStepMaster = vdata
    
End Property
Public Property Get NodeDB() As Database

    Set NodeDB = mdbsStepMaster
    
End Property

Private Sub CheckDupParameterName()
    ' Check if the parameter name already exists in the workspace
   
    Dim rstParameter As Recordset
    Dim strSql As String
    Dim qy As DAO.QueryDef

    On Error GoTo CheckDupParameterNameErr
    mstrSource = mstrModuleName & "CheckDupParameterName"
    
    ' Create a recordset object to retrieve the count of all parameters
    ' for the workspace with the same name
    strSql = "Select count(*) as parameter_count " & _
        " from workspace_parameters " & _
        " where workspace_id = [w_id]" & _
        " and parameter_name = [p_name]" & _
        " and parameter_id <> [p_id]"
    
    Set qy = mdbsStepMaster.CreateQueryDef(gstrEmptyString, strSql)
    Call AssignParameters(qy)
    
    Set rstParameter = qy.OpenRecordset(dbOpenForwardOnly)
    
    If rstParameter![parameter_count] > 0 Then
        rstParameter.Close
        qy.Close
        ShowError errDuplicateParameterName
        On Error GoTo 0
        Err.Raise vbObjectError + errDuplicateParameterName, _
            mstrSource, LoadResString(errDuplicateParameterName)
    End If
    
    rstParameter.Close
    qy.Close

    Exit Sub
    
CheckDupParameterNameErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "CheckDupParameterName"
    On Error GoTo 0
    Err.Raise vbObjectError + errCheckDupParameterNameFailed, _
        mstrSource, LoadResString(errCheckDupParameterNameFailed)

End Sub
Private Sub CheckDB()
    ' Check if the database object has been initialized
   
    If mdbsStepMaster Is Nothing Then
        On Error GoTo 0
        Err.Raise vbObjectError + errInvalidDB, _
            mstrModuleName & "CheckDB", LoadResString(errInvalidDB)
    End If

End Sub
Public Property Let ParameterValue(vdata As String)

    mstrParameterValue = vdata
    
End Property
Public Property Let Description(vdata As String)

    mstrDescription = vdata
    
End Property
Public Property Let ParameterType(vdata As ParameterType)

    mintParameterType = vdata
    
End Property

Public Property Let ParameterName(vdata As String)

    If vdata = gstrEmptyString Then
    
        ShowError errParameterNameMandatory
        On Error GoTo 0
        ' Propogate this error back to the caller
        Err.Raise vbObjectError + errParameterNameMandatory, _
            mstrSource, LoadResString(errParameterNameMandatory)
    Else
        mstrParameterName = vdata
    End If
    
End Property

Public Property Let ParameterId(vdata As Long)
    mlngParameterId = vdata
End Property

Public Property Let IndOperation(ByVal vdata As Operation)
    
    ' The valid operations are define in the cOperations
    ' class. Check if the operation is valid
    Select Case vdata
        Case QueryOp, InsertOp, UpdateOp, DeleteOp
            mintOperation = vdata
        
        Case Else
            On Error GoTo 0
            Err.Raise vbObjectError + errInvalidOperation, _
                    mstrSource, LoadResString(errInvalidOperation)
    End Select
    
End Property
Public Sub Validate()
    ' Each distinct object will have a Validate method which
    ' will check if the class properties are valid. This method
    ' will be used to check interdependant properties that
    ' cannot be validated by the let procedures.
    ' It should be called by the add and modify methods of the class
            
    On Error GoTo ValidateErr
    
    ' Check if the db object is valid
    Call CheckDB
    
    ' Call procedure to raise an error if the parameter name
    ' already exists in the workspace -
    ' if there are duplicates, we don't know what value for the
    ' parameter to use at runtime
    Call CheckDupParameterName

    Exit Sub
        
ValidateErr:

    mstrSource = mstrModuleName & "Validate"
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errValidateFailed, _
        mstrSource, LoadResString(errValidateFailed)
    
End Sub
Public Property Let WorkspaceId(vdata As Long)

    mlngWorkspaceId = vdata
    
End Property

Public Sub Add()

    Dim strInsert As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo AddErr
    
    ' Validate the record before trying to insert the record
    Call Validate
    
    ' Create a temporary querydef object
    strInsert = "insert into workspace_parameters " & _
            "( workspace_id, parameter_id, " & _
            " parameter_name, parameter_value, " & _
            " description, parameter_type ) " & _
            " values ( [w_id], [p_id], [p_name], [p_value], [desc], [p_type] ) "
    Set qy = mdbsStepMaster.CreateQueryDef(gstrEmptyString, strInsert)
    
    ' Call a procedure to assign the parameter values
    Call AssignParameters(qy)
    
    qy.Execute dbFailOnError
    qy.Close

'    strInsert = "insert into workspace_parameters " & _
'            "( workspace_id, parameter_id, " & _
'            " parameter_name, parameter_value ) " & _
'            " values ( " & _
'            Str(mlngWorkspaceId) & ", " & Str(mlngParameterId) & _
'            ", " & mFieldValue.MakeStringFieldValid(mstrParameterName) & _
'            ", " & mFieldValue.MakeStringFieldValid(mstrParameterValue) & " ) "
'    mdbsStepMaster.Execute strInsert, dbFailOnError + dbSQLPassThrough

    Exit Sub
        
AddErr:

    mstrSource = mstrModuleName & "Add"
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errParameterInsertFailed, _
        mstrSource, LoadResString(errParameterInsertFailed)
    
End Sub
Public Sub Delete()

    Dim strDelete As String
    Dim qy As DAO.QueryDef
        
    On Error GoTo DeleteErr
    
    ' Check if the db object is valid
    Call CheckDB
    
    strDelete = "delete from workspace_parameters " & _
            " where parameter_id = [p_id]"
    Set qy = mdbsStepMaster.CreateQueryDef(gstrEmptyString, strDelete)
    
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close
    
    Exit Sub
        
DeleteErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "Delete"
    On Error GoTo 0
    Err.Raise vbObjectError + errDeleteParameterFailed, _
            mstrSource, _
            LoadResString(errDeleteParameterFailed)
    
End Sub

Public Sub Modify()

    Dim strUpdate As String
    Dim qy As QueryDef
            
    On Error GoTo ModifyErr
    
    ' Validate the updated values before trying to modify the db
    Call Validate
    
    ' Create a temporary querydef object with the modify string
    strUpdate = "update workspace_parameters " & _
            " set workspace_id = [w_id], " & _
            "parameter_name = [p_name], " & _
            "parameter_value = [p_value], " & _
            "description = [desc], " & _
            "parameter_type = [p_type] " & _
            " where parameter_id = [p_id]"
    Set qy = mdbsStepMaster.CreateQueryDef(gstrEmptyString, strUpdate)
    
    ' Call a procedure to assign the parameter values to the
    ' querydef object
    Call AssignParameters(qy)
    qy.Execute dbFailOnError
    
    qy.Close

'    mdbsStepMaster.Execute strUpdate, dbFailOnError
'
    Exit Sub
        
ModifyErr:

    mstrSource = mstrModuleName & "Modify"
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errParameterUpdateFailed, _
        mstrSource, LoadResString(errParameterUpdateFailed)
    
End Sub
Public Property Get ParameterName() As String

    ParameterName = mstrParameterName
    
End Property

Public Property Get ParameterId() As Long

    ParameterId = mlngParameterId
    
End Property
Public Property Get NextIdentifier() As Long
    
    Dim lngNextId As Long

    On Error GoTo NextIdentifierErr
    
    ' First check if the database object is valid
    Call CheckDB
    
    ' Retrieve the next identifier using the sequence class
    Set mParameterSeq = New cSequence
    Set mParameterSeq.IdDatabase = mdbsStepMaster
    mParameterSeq.IdentifierColumn = FLD_ID_PARAMETER
    lngNextId = mParameterSeq.Identifier
    Set mParameterSeq = Nothing
    
    NextIdentifier = lngNextId
    Exit Property
    
NextIdentifierErr:
    LogErrors Errors
    mstrSource = mstrModuleName & "NextIdentifier"
    On Error GoTo 0
    Err.Raise vbObjectError + errIdGetFailed, _
        mstrSource, LoadResString(errIdGetFailed)
    
End Property
Public Property Get IndOperation() As Operation

    IndOperation = mintOperation
    
End Property

Public Property Get WorkspaceId() As Long

    WorkspaceId = mlngWorkspaceId
    
End Property

Public Property Get ParameterValue() As String

    ParameterValue = mstrParameterValue
    
End Property
Public Property Get Description() As String

    Description = mstrDescription
    
End Property
Public Property Get ParameterType() As ParameterType

    ParameterType = mintParameterType
    
End Property


Private Sub Class_Initialize()

    Set mFieldValue = New cStringSM
    
    ' Initialize the operation indicator variable to Query
    ' It will be modified later by the collection class when
    ' inserts, updates or deletes are performed
    mintOperation = QueryOp
    
End Sub

Private Sub Class_Terminate()

    Set mdbsStepMaster = Nothing
    Set mFieldValue = Nothing
    
End Sub
